/* REXXMP 0.11 (Jul 13 1999) - REXX Media Player
 * by Anton Dushko (2:5030/611.9)
 * This stuff is free. Comments & suggestions are welcome.
 */
                                         
/* Startup operations */
Trace Off
Address CMD
Signal On Error Name ShitHappens
Signal On Syntax Name ShitHappens
Signal On Halt Name CtrlBreak

/* Get arguments */
Parse Arg Media

/* Defaults */
StartStopped = 0
NoExitAtEnd = 0
NoRewindAtEnd = 0

/* Parse arguments */
Do While Left(Word(Media, 1), 1) = "/"
  Switch = Translate(SubStr(Word(Media, 1), 2))
  Select
    /* Start in stoppped state */
    When Switch = "S" Then
    Do
      StartStopped = 1
      NoExitAtEnd = 1
    End
    /* No exit after playing whole media */
    When Switch = "N" Then
      NoExitAtEnd = 1
    /* No rewind after playing whole media */
    When Switch = "R" Then
      NoRewindAtEnd = 1
    /* Change window size */
    When Switch = "W" Then
      "@MODE 32 8"
    Otherwise
      NOP
  End
  Media = SubWord(Media, 2)
End

/* Help */
If Media = "" Then Do
  Call LineOut , "REXXMP 0.1 (Mar 29 1999) - REXX Media Player" || "0A0D"x ||,
    "by Anton Dushko (2:5030/611.9)" || "0A0D0A0D"x ||,
    "Usage: REXXMP [space separated switches] <media file or device name>" || "0A0D"x ||,
    "Switches:" || "0A0D"x ||,
    "  /S - Start in stopped state (no autoplay)" || "0A0D"x ||,
    "  /N - Don't exit after playing whole media even if no keys was pressed" || "0A0D"x ||,
    "  /R - Don't rewind after playing whole media" || "0A0D"x ||,
    "  /W - Change window size to match player's face size"
  Exit 1
End

/* Face */
Call SysCurState "OFF"
Call CharOut "CON", "[=7l"||,
  "Ŀ" || "0A0D"x ||,
  "                             " || "0A0D"x ||,
  "        Loading  " || "0A0D"x ||,
  "                              " || "0A0D"x ||,
  " [                    ] --:-- " || "0A0D"x ||,
  "Ĵ" || "0A0D"x ||,
  " <<  S  >  P  >>     M " || "0A0D"x ||,
  "[7A[32D"

/* Load & initialize MCI REXX API */
Call RxFuncAdd "mciRxInit", "MCIAPI", "mciRxInit"
If RxFuncQuery("mciRxInit") \= 0 Then
Do
  Call CharOut "CON", "[s[2B[21C[1;33mNo MMOS/2[u[6B" || "0A0D"x
  Call SysCurState "ON"
  Exit -3
End
Call mciRxInit

/* Initialize big digits */
BigDigits.0 = "[1B[3D [1B[3D"
BigDigits.1 = " [1B[3D  [1B[3D"
BigDigits.2 = "[1B[3D[1B[3D"
BigDigits.3 = "[1B[3D [1B[3D"
BigDigits.4 = " [1B[3D[1B[3D  "
BigDigits.5 = "[1B[3D[1B[3D"
BigDigits.6 = "[1B[3D[1B[3D"
BigDigits.7 = "[1B[3D  [1B[3D  "
BigDigits.8 = "[1B[3D[1B[3D"
BigDigits.9 = "[1B[3D[1B[3D"

/* Open media */
Call CharOut "CON", "[s[2B[21C[1;33m Opening [u"
Call SendMCIStr "OPEN" Media "SHAREABLE WAIT"

/* Set media time format */
Call CharOut "CON", "[s[2B[21C[1;33mStarting [u"
Call SendMCIStr "SET" Media "TIME FORMAT MS WAIT"

/* Get, print & fix media length */
Media.Length = SendMCIStr("STATUS" Media "LENGTH WAIT") % 1000
Call CharOut "CON", "[s[4B[25C[1;36m"Right(Media.Length % 60, 2, 0)":"Right(Media.Length // 60, 2, 0)"[u"
If Media.Length = 0 Then Media.Length = 1

/* Determine media type */
Media.Type = Translate(SendMCIStr("CAPABILITY" Media "DEVICE TYPE WAIT"))
Select
  When Media.Type = "WAVEAUDIO"    Then Call CharOut "CON", "[s[B[23C[1;31mWave[u"
  When Media.Type = "SEQUENCER"    Then Call CharOut "CON", "[s[B[23C[1;31mMIDI[u"
  When Media.Type = "CDAUDIO"      Then Call CharOut "CON", "[s[B[24C[1;31mCD[u"
  When Media.Type = "DIGITALVIDEO" Then Call CharOut "CON", "[s[B[23C[1;31mVideo[u"
  When Media.Type = "VIDEODISC"    Then Call CharOut "CON", "[s[B[23C[1;31mVideo[u"
  When Media.Type = "OVERLAY"      Then Call CharOut "CON", "[s[B[23C[1;31mVideo[u"
  When Media.Type = "ANIMATION"    Then Call CharOut "CON", "[s[B[23C[1;31mVideo[u"
  Otherwise                             Call CharOut "CON", "[s[B[23C[1;31mOther[u"
End

/* Play media*/
If \StartStopped Then
  Call SendMCIStr "PLAY" Media

/* Big time */
Call PutBigDigit 2, 1, 0
Call PutBigDigit 6, 1, 0
Call PutBigDigit 12, 1, 0
Call PutBigDigit 16, 1, 0

/* Main loop */ 
BlinkDotsOn = 1
Media.Muted = 0
Media.OldMode = ""
HaveAnyKeyBeenPressedSinceStart = 0
Do Forever

  /* Get media status */
  Media.Mode = Translate(SendMCIStr("STATUS" Media "MODE WAIT"))
  Media.Position = SendMCIStr("STATUS" Media "POSITION WAIT") % 1000
  Parse Value SendMCIStr("STATUS" Media "VOLUME WAIT") With Media.Volume":".

  /* Big time */
  Call PutBigDigit 2, 1, (Media.Position // 6000) % 600
  Call PutBigDigit 6, 1, (Media.Position // 600) % 60
  Call PutBigDigit 12, 1, (Media.Position // 60) % 10
  Call PutBigDigit 16, 1, Media.Position // 10

  /* Slider */
  SliderPos = Media.Position / Media.Length
  Select
    When SliderPos > 1 Then SliderPos = 1
    When SliderPos < 0 Then SliderPos = 0
    Otherwise NOP
  End
  SliderPos = Format(SliderPos * 39,, 0)
  If SliderPos // 2 Then
    Call CharOut "CON", "[s[4B[2C[1;36m[" || Overlay("", "                    ", 1 + SliderPos % 2) || "][u"
  Else
    Call CharOut "CON", "[s[4B[2C[1;36m[" || Overlay("", "                    ", 1 + SliderPos % 2) || "][u"

  /* Status */
  Select
    When Media.Mode = "PLAYING" Then
      Call CharOut "CON", "[s[2B[21C[1;33m Playing [u"
    When Media.Mode = "PAUSED" Then
      Call CharOut "CON", "[s[2B[21C[1;33m Paused  [u"
    When Media.Mode = "STOPPED" Then
    Do
      Call CharOut "CON", "[s[2B[21C[1;33m Stopped [u"
      /* Exit at the end of media */
      If \NoExitAtEnd & \HaveAnyKeyBeenPressedSinceStart Then
        Leave
      /* Rewind at the end of media */
      If \NoRewindAtEnd & (Media.OldMode = "PLAYING") & (Char = "") Then
        Call SendMCIStr "SEEK" Media "TO START"
    End
    When Media.Mode = "NOT READY" Then
      Call CharOut "CON", "[s[2B[21C[1;33mNot ready[u"
    When Media.Mode = "SEEKING" Then
      Call CharOut "CON", "[s[2B[21C[1;33m Seeking [u"
    When Media.Mode = "OTHER" Then
      Call CharOut "CON", "[s[2B[21C[1;33m  Other  [u"
    Otherwise
      Call CharOut "CON", "[s[2B[21C[1;33m Unknown [u"
  End
  Media.OldMode = Media.Mode

  /* Volume */
  If Media.Muted Then
    Call CharOut "CON", "[s[3B[21C[1;30m[ Muted ][u"
  Else
    Call CharOut "CON", "[s[3B[21C[1;30mVol: "|| Left(Media.Volume || "%", 4) || "[u"

  /* Blinking dots */
  If BlinkDotsOn Then
    Call CharOut "CON", "[s[1B[10C[1;32m[1B[1D[u"
  Else
    Call CharOut "CON", "[s[1B[10C [1B[1D [u"
  BlinkDotsOn = \BlinkDotsOn

  /* Non-blocking input*/
  If Chars() > 0 Then
  Do
    HaveAnyKeyBeenPressedSinceStart = 1
    Char = SysGetKey("NOECHO")
    If Char = "E0"x Then
      Char = Char || (SysGetKey("NOECHO"))
    Else
      Char = Translate(Char)
  End
  Else
  Do
    Char = ""
    Call SysSleep 1
  End

  /* User interaction*/
  Select
    /* Play/Stop */
    When Char = "X" Then Do
      Call CharOut "CON", "[s[6B[6C[1;37m[S][u"
      Call SendMCIStr "STOP" Media
    End
    When Char = "C" Then Do
      Call CharOut "CON", "[s[6B[10C[1;37m[>][u"
      Call SendMCIStr "PLAY" Media
    End

    /* Pause/Resume */
    When Char = "V" Then Do
      Call CharOut "CON", "[s[6B[14C[1;37m[P][u"
      Select
        When Media.Mode = "PAUSED" Then
          Call SendMCIStr "RESUME" Media
        Otherwise
          Call SendMCIStr "PAUSE" Media
      End
    End

    /* Seeks */
    When Char = "E04B"x Then Do
      Call CharOut "CON", "[s[6B[C[1;37m[<<][u"
      Call SendMCIStr "SEEK" Media "TO" Max(Min(Media.Position - 10, Media.Length), 0) * 1000
    End
    When Char = "E04D"x Then Do
      Call CharOut "CON", "[s[6B[18C[1;37m[>>][u"
      Call SendMCIStr "SEEK" Media "TO" Min(Media.Position + 10, Media.Length) * 1000
    End
    When Char = "Z" Then Do
      Call CharOut "CON", "[s[6B[C[1;37m[|<][u"
      Call SendMCIStr "SEEK" Media "TO START"
    End
    When Char = "B" Then Do
      Call CharOut "CON", "[s[6B[18C[1;37m[>|][u"
      Call SendMCIStr "SEEK" Media "TO END"
    End

    /* Volume */
    When Char = "M" Then Do
      Call CharOut "CON", "[s[6B[28C[1;37m[M][u"
      If Media.Muted Then
        Call SendMCIStr "SET" Media "AUDIO ON"
      Else
        Call SendMCIStr "SET" Media "AUDIO OFF"
      Media.Muted = \Media.Muted
    End
    When Char = "E048"x Then Do
      Call CharOut "CON", "[s[6B[23C[1;37m[][u"
      Call SendMCIStr "SET" Media "AUDIO VOLUME" Min(Media.Volume + 10, 100)
    End
    When Char = "E050"x Then Do
      Call CharOut "CON", "[s[6B[25C[1;37m[][u"
      Call SendMCIStr "SET" Media "AUDIO VOLUME" Max(Media.Volume - 10, 0)
    End

    /* Exit */
    When Char = "1B"x Then
    Do
      Call CharOut "CON", "[s[2B[21C[1;33m  Done!  [u"
      Call SendMCIStr "STOP" Media
      Leave
    End

    /* Ignore other */
    Otherwise
      Call CharOut "CON", "[s[6B[C[0m <<  S  >  P  >>     M [u"
  End
End

/* Close media*/
Call SendMCIStr "CLOSE" Media "WAIT"

/* Deinitialize MCI REXX API */
Call mciRxExit

/* Close face */
Call CharOut "CON", "[6B" || "0A0D"x
Call SysCurState "ON"

/* All OK */
Exit 0

/* Send MCI Command String */
SendMCIStr: Procedure
  Parse Arg CommandStr
  RC = mciRxSendString(CommandStr, "ReturnStr", 0, 0)
  If RC \= 0 Then
    Call CharOut "CON", "[s[2B[21C[1;31mMCI Error[u"
Return ReturnStr

/* Type a big digit */
PutBigDigit: Procedure Expose BigDigits.
  Parse Arg X, Y, Digit
  Call CharOut "CON", "[s["|| X ||"C["|| Y ||"B[1;32m"|| BigDigits.Digit || "[u"
Return

/* Signal handlers */

ShitHappens:
  Call CharOut "CON", "[s[2B[21C[1;31mInt Error[u"
  Call mciRxExit
  Call CharOut "CON", "[6B" || "0A0D"x
  Call SysCurState "ON"
Exit -2

CtrlBreak:
  Call CharOut "CON", "[s[2B[21C[1;33mUsr Break[u"
  Call mciRxExit
  Call CharOut "CON", "[6B" || "0A0D"x
  Call SysCurState "ON"
Exit -1

/* End of REXXMP.CMD */
